@@ -20,10 +20,23 @@ enum {
 	LAST_SIGNAL
 };
 
+enum {
+        MARKER_NORMAL = 0,
+        MARKER_HILIGHT,
+        MARKER_CURRENT,
+        MARKER_NB
+};
+
+static char *marker_files[MARKER_NB] = {
+        ICONDIR "/clock-map-location-marker.png",
+        ICONDIR "/clock-map-location-hilight.png",
+        ICONDIR "/clock-map-location-current.png"
+};
+
 static guint signals[LAST_SIGNAL];
 
 typedef struct {
-        struct tm last_refresh;
+        time_t last_refresh;
 
         gint width;
         gint height;
@@ -31,7 +44,7 @@ typedef struct {
 	guint highlight_timeout_id;
 
         GdkPixbuf *stock_map_pixbuf;
-        GdkPixbuf *location_marker_pixbuf[3];
+        GdkPixbuf *location_marker_pixbuf[MARKER_NB];
 
         GdkPixbuf *location_map_pixbuf;
 
@@ -40,9 +53,6 @@ typedef struct {
 
         /* The map with the shadow composited onto it */
         GdkPixbuf *shadow_map_pixbuf;
-
-        /* The rotated shadow_map_pixbuf, ready for display */
-        GdkPixbuf *rotated_map_pixbuf;
 } ClockMapPrivate;
 
 static void clock_map_finalize (GObject *);
@@ -55,7 +65,6 @@ static gboolean clock_map_expose (GtkWidget *this,
 
 static void clock_map_place_locations (ClockMap *this);
 static void clock_map_render_shadow (ClockMap *this);
-static void clock_map_rotate (ClockMap *this);
 static void clock_map_display (ClockMap *this);
 
 #define PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CLOCK_MAP_TYPE, ClockMapPrivate))
@@ -64,19 +73,8 @@ ClockMap *
 clock_map_new (void)
 {
         ClockMap *this;
-        ClockMapPrivate *priv;
 
         this = g_object_new (CLOCK_MAP_TYPE, NULL);
-        priv = PRIVATE (this);
-
-        priv->location_marker_pixbuf[0] = gdk_pixbuf_new_from_file
-                (ICONDIR "/clock-map-location-marker.png", NULL);
-        priv->location_marker_pixbuf[1] = gdk_pixbuf_new_from_file
-                (ICONDIR "/clock-map-location-hilight.png", NULL);
-        priv->location_marker_pixbuf[2] = gdk_pixbuf_new_from_file
-                (ICONDIR "/clock-map-location-current.png", NULL);
-
-        clock_map_refresh (this);
 
         return this;
 }
@@ -119,15 +117,23 @@ clock_map_class_init (ClockMapClass *this_class)
 static void
 clock_map_init (ClockMap *this)
 {
+        int i;
         ClockMapPrivate *priv = PRIVATE (this);
 
 	GTK_WIDGET_SET_FLAGS (this, GTK_NO_WINDOW);
 
+	priv->last_refresh = 0;
+	priv->width = 0;
+	priv->height = 0;
 	priv->highlight_timeout_id = 0;
         priv->stock_map_pixbuf = NULL;
-        priv->location_marker_pixbuf[0] = NULL;
-        priv->location_marker_pixbuf[1] = NULL;
-        priv->location_marker_pixbuf[2] = NULL;
+
+        g_assert (sizeof (marker_files)/sizeof (char *) == MARKER_NB);
+
+        for (i = 0; i < MARKER_NB; i++) {
+                priv->location_marker_pixbuf[i] = gdk_pixbuf_new_from_file
+                                                  (marker_files[i], NULL);
+        }
 }
 
 static void
@@ -146,7 +152,7 @@ clock_map_finalize (GObject *g_obj)
                 priv->stock_map_pixbuf = NULL;
         }
 
-	for (i = 0; i < 3; i++) {
+	for (i = 0; i < MARKER_NB; i++) {
         	if (priv->location_marker_pixbuf[i]) {
                 	gdk_pixbuf_unref (priv->location_marker_pixbuf[i]);
                 	priv->location_marker_pixbuf[i] = NULL;
@@ -168,11 +174,6 @@ clock_map_finalize (GObject *g_obj)
                 priv->shadow_map_pixbuf = NULL;
         }
 
-        if (priv->rotated_map_pixbuf) {
-                gdk_pixbuf_unref (priv->rotated_map_pixbuf);
-                priv->rotated_map_pixbuf = NULL;
-        }
-
         G_OBJECT_CLASS (clock_map_parent_class)->finalize (g_obj);
 }
 
@@ -181,34 +182,28 @@ clock_map_refresh (ClockMap *this)
 {
         ClockMapPrivate *priv = PRIVATE (this);
 	GtkWidget *widget = GTK_WIDGET (this);
-        GtkWidget *parent = gtk_widget_get_parent (widget);
-	GtkRequisition req;
-        gint width;
-        gint height;
 
-	gtk_widget_size_request (widget, &req);
-        width = req.width;
-        height = req.height;
-
-        if (parent) {
-		if (widget->allocation.width != 1) {
-			width = widget->allocation.width;
-			height = widget->allocation.height;
-		}
+        /* Only do something if we have some space allocated.
+         * Note that 1x1 is not really some space... */
+        if (widget->allocation.width <= 1 || widget->allocation.height <= 1)
+                return;
 
+        /* Allocation changed => we reload the map */
+	if (priv->width != widget->allocation.width ||
+	    priv->height != widget->allocation.height) {
                 if (priv->stock_map_pixbuf) {
                         gdk_pixbuf_unref (priv->stock_map_pixbuf);
                         priv->stock_map_pixbuf = NULL;
                 }
+
+                priv->width = widget->allocation.width;
+                priv->height = widget->allocation.height;
         }
 
         if (!priv->stock_map_pixbuf) {
                 GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale
                         (ICONDIR "/clock-map.png",
-                         width, height, FALSE, NULL);
-
-                priv->width = gdk_pixbuf_get_width (pixbuf);
-                priv->height = gdk_pixbuf_get_height (pixbuf);
+                         priv->width, priv->height, FALSE, NULL);
 
                 priv->stock_map_pixbuf = pixbuf;
         }
@@ -226,20 +221,22 @@ clock_map_expose (GtkWidget *this, GdkEventExpose *event)
 	GdkRectangle region;
         cairo_t *cr;
 
-	if (!priv->rotated_map_pixbuf)
+	if (!priv->shadow_map_pixbuf) {
+                g_warning ("Needed to refresh the map in expose event.");
 		clock_map_refresh (CLOCK_MAP (this));
+        }
 
         cr = gdk_cairo_create (this->window);
 
 	region.x = allocation.x;
 	region.y = allocation.y;
-	region.width = gdk_pixbuf_get_width (priv->rotated_map_pixbuf);
-	region.height = gdk_pixbuf_get_height (priv->rotated_map_pixbuf);
+	region.width = gdk_pixbuf_get_width (priv->shadow_map_pixbuf);
+	region.height = gdk_pixbuf_get_height (priv->shadow_map_pixbuf);
 
 	gdk_rectangle_intersect (&region, &(event->area), &region);
 	gdk_draw_pixbuf (this->window,
 			 this->style->black_gc,
-			 priv->rotated_map_pixbuf,
+			 priv->shadow_map_pixbuf,
 			 region.x - allocation.x,
 			 region.y - allocation.y,
 			 region.x,
@@ -253,8 +250,8 @@ clock_map_expose (GtkWidget *this, GdkEventExpose *event)
         cairo_rectangle (
                 cr,
                 allocation.x + 0.5, allocation.y + 0.5,
-                gdk_pixbuf_get_width (priv->rotated_map_pixbuf) - 1,
-                gdk_pixbuf_get_height (priv->rotated_map_pixbuf) - 1);
+                gdk_pixbuf_get_width (priv->shadow_map_pixbuf) - 1,
+                gdk_pixbuf_get_height (priv->shadow_map_pixbuf) - 1);
 
         cairo_set_source_rgb (
                 cr,
@@ -285,9 +282,8 @@ clock_map_size_allocate (GtkWidget *this, GtkAllocation *allocation)
 	if (GTK_WIDGET_CLASS (clock_map_parent_class)->size_allocate)
 		GTK_WIDGET_CLASS (clock_map_parent_class)->size_allocate (this, allocation);
 
-	if (!priv->stock_map_pixbuf
-	    || priv->width != this->allocation.width
-	    || priv->height != this->allocation.height)
+	if (priv->width != this->allocation.width ||
+	    priv->height != this->allocation.height)
                 clock_map_refresh (CLOCK_MAP (this));
 }
 
@@ -419,11 +415,11 @@ clock_map_place_location (ClockMap *this, ClockLocation *loc, gboolean hilight)
         clock_location_get_coords (loc, &latitude, &longitude);
 
 	if (hilight)
-		marker = 1;
+		marker = MARKER_HILIGHT;
 	else if (clock_location_is_current (loc))
-		marker = 2;
+		marker = MARKER_CURRENT;
 	else
-		marker = 0;
+		marker = MARKER_NORMAL;
 
         clock_map_mark (this, latitude, longitude, marker);
 }
@@ -584,78 +580,14 @@ clock_map_render_shadow (ClockMap *this)
 }
 
 static void
-clock_map_rotate (ClockMap *this)
+clock_map_display (ClockMap *this)
 {
         ClockMapPrivate *priv = PRIVATE (this);
 
-        int rot_center, map_center;
-        gfloat latitude, longitude;
-
-        map_center = priv->width / 2;
-
-#ifdef TESTER
-	priv->rotated_map_pixbuf = gdk_pixbuf_copy (priv->shadow_map_pixbuf);
-
-#else
-
-        if (priv->rotated_map_pixbuf) {
-                gdk_pixbuf_unref (priv->rotated_map_pixbuf);
-        }
-
-        priv->rotated_map_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-                                                   priv->width, priv->height);
-
-        latitude = 0.0;
-        longitude = 0.0;
-
-        rot_center = (priv->width / 2.0 + (priv->width / 2.0) * longitude / 180.0);
-
-        if (rot_center < map_center) {
-                /* copy the part after the image center */
-                gdk_pixbuf_copy_area (priv->shadow_map_pixbuf,
-                                      rot_center, 0,
-                                      map_center, priv->height,
-                                      priv->rotated_map_pixbuf, map_center, 0);
-
-                /* copy the part before the image center */
-                gdk_pixbuf_copy_area (priv->shadow_map_pixbuf,
-                                      rot_center + map_center, 0,
-                                      priv->width - (rot_center + map_center),
-                                      priv->height,
-                                      priv->rotated_map_pixbuf, 0, 0);
-
-                gdk_pixbuf_copy_area (priv->shadow_map_pixbuf,
-                                      0, 0, rot_center, priv->height,
-                                      priv->rotated_map_pixbuf,
-                                      map_center - rot_center, 0);
-        } else {
-                /* copy the part after the image center */
-                gdk_pixbuf_copy_area (priv->shadow_map_pixbuf,
-                                      rot_center - map_center, 0,
-                                      map_center, priv->height,
-                                      priv->rotated_map_pixbuf, 0, 0);
-
-                /* copy the part before the image center */
-                gdk_pixbuf_copy_area (priv->shadow_map_pixbuf,
-                                      rot_center, 0, priv->width - rot_center,
-                                      priv->height,
-                                      priv->rotated_map_pixbuf, map_center, 0);
-
-                gdk_pixbuf_copy_area (priv->shadow_map_pixbuf,
-                                      0, 0, rot_center - map_center,
-                                      priv->height,
-                                      priv->rotated_map_pixbuf,
-                                      map_center + priv->width -  rot_center, 0);
-        }
-#endif
-}
-
-static void
-clock_map_display (ClockMap *this)
-{
         clock_map_render_shadow (this);
-        clock_map_rotate (this);
 	gtk_widget_queue_draw (GTK_WIDGET (this));
+
+        time (&priv->last_refresh);
 }
 
 typedef struct {
@@ -727,26 +659,12 @@ static gboolean
 clock_map_needs_refresh (ClockMap *this)
 {
         ClockMapPrivate *priv = PRIVATE (this);
-        gboolean refresh = FALSE;
-
-        struct tm now;
         time_t now_t;
+
         time (&now_t);
 
-        gmtime_r (&now_t, &now);
-
-        /* refresh once per minute */
-        if (now.tm_year > priv->last_refresh.tm_year
-            || now.tm_mon > priv->last_refresh.tm_mon
-            || now.tm_mday > priv->last_refresh.tm_mday
-            || now.tm_hour > priv->last_refresh.tm_hour
-            || now.tm_min > priv->last_refresh.tm_min) {
-                refresh = TRUE;
-        }
-
-        gmtime_r (&now_t, &priv->last_refresh);
-
-        return refresh;
+	/* refresh once per minute */
+	return (ABS (now_t - priv->last_refresh) >= 60);
 }
 
 void
